//////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2007-2011 Infinite Z, Inc.  All Rights Reserved.
//
//  File:       HeadTrackingGlutWindow.h
//  Content:    Sample code demonstrating use of 
//				      ZSStereo's Head Tracking in Default (simple) mode.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __ZSPACE_STEREO_GLUT_WINDOW_H__
#define __ZSPACE_STEREO_GLUT_WINDOW_H__

#include <freeglut.h>	//"GL/glut.h"	//S3D
#include "Scene.h"
#include "ZSpaceInterface.h"


// wrapper class for GLUT window callback functions
//
// Needs to be a singleton/static because GLUT callbacks do not have a
//  mechanism to say which window is attached to the callback.
//
// Alternately, could use one global callback for each callback function
//  and use glutGetWindow()'s ID to map to an object.
class HeadTrackingGlutWindow
{
  static char const * const WINDOW_NAME;
public:
  static HeadTrackingGlutWindow* GetInstance();
  // Initialize the window.
private:
  static HeadTrackingGlutWindow* s_instance;
  HeadTrackingGlutWindow();
  ~HeadTrackingGlutWindow();

public:
  int initialize(int windowX, int windowY, int windowWidth, int windowHeight);

  // Clean up resources and (HACK!) delete instance
  // Don't want to write a singleton destroyer for an example app
  static void Clean();

/////////////////////////////////////
// GLUT callback system

private:
  static void RegisterCallbacks();

  // static callback wrappers
  static void GlutDisplayFunc();
  static void GlutKeyboardFunc(unsigned char key, int x, int y);
  static void GlutReshapeFunc(GLint width, GLint height);
  static void GlutIdleFunc();
  // ZSpace needs to know when the window moves, but GLUT
  //  does not have a window move callback
  // static void GlutMoveFunc(GLint x, GLint y);

  // GLUT member functions (and a fakie)
  void glutDisplayFunc();
  void glutKeyboardFunc(unsigned char key, int x, int y);
  void glutReshapeFunc(int width, int height);
  void glutIdleFunc();
  void moveFunc(int x, int y);

// GLUT callback system
/////////////////////////////////////

/////////////////////////////////////
// Display rendering functionality

public:
  void setScene(Scene* scene) { m_scene = scene; }
  Scene* getScene() { return m_scene; }

  // Returns a gluLookAt compatible position, direction and up vector
  //  such that the object space origin (0,0,0) will show up in the
  //  center of the viewport on the display plane.
  void getDefaultCameraPosition(float* pos3look3up3);

  // Single frame rendering function.
  void renderFrame();

private:
  void renderScene();
  Scene* m_scene;

// Display rendering functionality
/////////////////////////////////////

/////////////////////////////////////
// ZSpace additions

public:
  int initializeEyes();
  int initializeStereo(int windowX, int windowY, int windowWidth, int windowHeight);

private:
  ZSpaceInterface* m_zSpaceInterface;
  // Set up modelview matrix, projection matrix, and render buffer for specified eye
  void renderEye(zspace::stereo::StereoFrustum::StereoEye eye);

// ZSpace additions
/////////////////////////////////////

/////////////////////////////////////
// miscellaneous window management
// So far this is only used for detecting when the
//  window moves
public:
  int getWindowX()      const;
  int getWindowY()      const;
  int getWindowWidth()  const;
  int getWindowHeight() const;
};


void SetGlutWindow(Scene* glutWindow);

#endif // __ZSPACE_STEREO_GLUT_WINDOW_H__